//havok_config.cpp
#include "MakeRigidBody.h"
#include <Common/Base/hkBase.h>
#include <Common/Base/Container/String/hkStringBuf.h>
#include <Common/Base/Ext/hkBaseExt.h>
#include <Common/Base/Memory/System/hkMemorySystem.h>
#include <Common/Base/Memory/System/Util/hkMemoryInitUtil.h>
#include <Common/Base/Memory/Allocator/Malloc/hkMallocAllocator.h>

#include <Physics/Dynamics/World/hkpWorld.h>
#include <Physics/Dynamics/Entity/hkpRigidBody.h>
#include <Physics/Dynamics/Entity/hkpRigidBodyCinfo.h>
#include <Physics/Utilities/Dynamics/Inertia/hkpInertiaTensorComputer.h>
#include <Physics/Collide/Shape/Convex/Box/hkpBoxShape.h>
#include <Physics/Collide/Dispatch/hkpAgentRegisterUtil.h>
#include <Physics/Collide/Filter/Group/hkpGroupFilter.h>
#include <Common/Serialize/Util/hkSerializeDeprecated.h>

#include "HavokManager.h"

const float g_groundHalfHeight = 2.0f;
//ڽ    
const float g_bx = 1.0f, g_by = 2.0f, g_bz = 1.0f;
const int g_nx = 2, g_ny = 2, g_nz = 2;
const int g_NUM_BODIES = g_nx * g_ny * g_nz;
hkpRigidBody *g_bodies[g_NUM_BODIES];


hkpRigidBody* BuildGround( hkpWorld* pkWorld )
{
	pkWorld->markForWrite();

    const hkVector4 halfExtents(2000.0f, g_groundHalfHeight, 2000.0f);
    hkpBoxShape* groundShape = new hkpBoxShape(halfExtents);

    hkpRigidBodyCinfo bodyInfo;
    bodyInfo.m_mass = 0.0f; 
    bodyInfo.m_shape = groundShape;
    bodyInfo.m_motionType = hkpMotion::MOTION_FIXED;
    bodyInfo.m_position.set(0.0f, -g_groundHalfHeight, 0.0f);
	//bodyInfo.m_collisionFilterInfo = HavokManager::LAYER_STATIC_BACKGROUND;

    hkpRigidBody* rigidBody = new hkpRigidBody(bodyInfo);
    groundShape->removeReference();

    pkWorld->addEntity( rigidBody );
    rigidBody->removeReference(); 

	pkWorld->unmarkForWrite();
	return rigidBody;
}

hkpRigidBody* BuildBox( hkpWorld* pkWorld )
{
	pkWorld->markForWrite();

    const hkVector4 halfExtents(g_bx, g_by, g_bz);
    hkpShape* shape = new hkpBoxShape(halfExtents, 0.0f);

    hkpMassProperties massProperties;
    hkpInertiaTensorComputer::computeShapeVolumeMassProperties(shape, 5.0f, massProperties);

    hkpRigidBodyCinfo bodyInfo;
    bodyInfo.m_mass = massProperties.m_mass;
    bodyInfo.m_centerOfMass = massProperties.m_centerOfMass;
    bodyInfo.m_inertiaTensor = massProperties.m_inertiaTensor;
    bodyInfo.m_shape = shape;
    bodyInfo.m_motionType = hkpMotion::MOTION_BOX_INERTIA;
	//bodyInfo.m_collisionFilterInfo = HavokManager::LAYER_ACTOR;

	hkpRigidBody* rigidBody = 0L;	
    int i = 0;
    for(int x = 0; x < g_nx; x++)
    for(int y = 0; y < g_ny; y++)
    for(int z = 0; z < g_nz; z++) 
	{
        hkpRigidBody *body = new hkpRigidBody(bodyInfo);
		g_bodies[i] = body;

        body->setPosition(hkVector4(x * g_bx * 2, 100.0f + y * g_by * 2, z * g_bz * 2));
        pkWorld->addEntity(body);
        body->removeReference();
        i++;
    }
    shape->removeReference();

	pkWorld->unmarkForWrite();
	return g_bodies[0];
}
